#include <stdlib.h>
#include <stdio.h>
#include <string.h>

typedef struct { char *str; void* next; } REC;
typedef struct { REC *first; int num; } LIST;

int list_init(LIST *list)
{
    list->first = NULL;
    list->num = 0;
    return 0;
}

int list_free(LIST *list)
{
    int n = 0;
    REC *p, *q;
    
    p = q = list->first;
    while(p)
    {
        free(p->str);
        q = p->next;
        free(p);
        p = q;
        n++;
    }
    list_init(list);
    
    return n;
}

REC *rec_get(LIST *list, int pos)
{
    REC *p;
    int i;
    
    p = list->first;
    i = 0;
    for(i = 0; i < pos; i++)
        p = p->next;
    return p;
}

int list_length(LIST *list)
{
    int n = 0;
    REC *p;
    
    p = list->first;
    while(p)
    {
        n++;
        p = p->next;
    }
    list->num = n;    
    return n;
}

char *str_get(LIST *list, int pos)
{
    return rec_get(list, pos)->str;
}

int str_pos(LIST *list, char *str)
{
    int pos = 0;
    int found = -1;
    
    for(pos = 0; pos < list->num; pos++)
    {
        if(strcmp(str_get(list, pos), str) == 0)
        {
            found = pos;
            pos = list->num;
        }
    }
    return found;
}

char *word_get(LIST *list, int strpos, int wordpos, char *word, int len)
{
    char *str, *p;
    int i;

    p = str = str_get(list, strpos);
    for(i = 0; i < wordpos; i++)
    {
        p = (char *)strchr(p, ' ');
        if(p)
            p++;
    }

    if(p)
    {
        strncpy(word, p, len);
        word[len - 1] = '\0';
        if(p = (char *)strchr(word, ' '))
            *p = '\0';
        return word;
    }

    return NULL;
}

int word_pos(LIST *list, int strpos, char *word)
{
    char *str, *off, *p;
    int i = 0;
    int found = -1;

    str = str_get(list, strpos);
    off = strstr(str, word);
    if(off)
    {
        found = -2;
        p = str;
        while(p && found < 0)
        {
            if(p == off)
                found = i;
            p = strchr(p, ' ');
            if(p)
                p++;
            i++;
        }
    }
    return found;
}

int head_pos(LIST *list, char *word)
{
    int i = 0;
    int pos;
    int found = -1;
    int nlist = list->num;

    while(i < nlist && found < 0)
    {
        pos = word_pos(list, i, word);
        if(pos == 0)
            found = i;
        i++;
    }

    return found;
}

int str_put(LIST *list, char *str, int pos)
{
    strcpy(rec_get(list, pos)->str, str);
    return pos;
}

int str_append(LIST *list, char *str)
{
    REC *p, *q;

    p = q = list->first;
    while(p)
    {
        q = p;
        p = p->next;
    }

    if(q)
    {
        q->next = (REC *)malloc(sizeof(REC));
        ((REC *)q->next)->str = (char *)malloc(strlen(str) + 1);
        strcpy(((REC *)q->next)->str, str);
        ((REC *)q->next)->next = NULL;
        list->num++;
    }
    else
    {
        p = (REC *)malloc(sizeof(REC));
        p->str = (char *)malloc(strlen(str) + 1);
        strcpy(p->str, str);
        p->next = NULL;
        list->first = p;
        list->num = 1;
    }
    return list_length(list);
}

int str_delete(LIST *list, int pos)
{
    REC *p, *q;
    
    if(pos)
    {
        p = rec_get(list, pos - 1);
        q = rec_get(list, pos);
        p->next = q->next;
        free(q->str);
        free(q);
        list->num--;
    }
    else
    {
        p = rec_get(list, 0);
        list->first = p->next;
        list->num--;
        free(p->str);
        free(p);
    }
    
    return list_length(list);
}

int list_read(LIST *list, char *file)
{
    int n = 0;
    FILE *f;
    char line[1024];
    char *pstr;

    if(f = fopen(file, "r"))
    {
        while(fgets(line, sizeof(line), f))
        {
            if(pstr = (char*)strchr(line, '\n'))
                *pstr = '\0';
            str_append(list, line);
            n++;
        }
        fclose(f);
    }
    return n;
}

int list_write(LIST *list, char *file)
{
    int i;
    FILE *f;
    
    if(f = fopen(file, "w"))
    {
        for(i = 0; i < list->num; i++)
            fprintf(f, "%s\n", str_get(list, i));
        fclose(f);
    }

    return list->num;
}


int main(void)
{
    /* input lists: */
    LIST deps; // all packages with dependencies
    LIST inst; // installed packages, with or without dependencies
    LIST wish; // packages to be checked, with or without dependencies

    /* output lists: */
    LIST done; // resolved wishlist, install the packages in that order
    LIST imp; // impossible to resolve, deps file has errors
    LIST rest; // still unresolved, should be empty after prog run
    LIST result;

    char pkg[1024]; // package name
    char tmp[1024];
    char *depword;
    int isdone;
    int isimp;
    int numrest;
    int irest, iwish, iinst, idone, ideps, depcol, iresult;
    int numdone;
    int i;
    int pos;
    
    list_init(&deps);
    list_init(&inst);
    list_init(&wish);
    list_init(&rest);
    list_init(&done);
    list_init(&imp);
    list_init(&result);
    
    /* read input files */
    list_read(&deps, "deps.txt");
    if(!deps.num)
    	printf("depsort needs file deps.txt (available packages with dependencies)\n");
    list_read(&inst, "inst.txt");
    list_read(&wish, "wish.txt");
    printf("lines read: deps=%d inst:=%d wish:=%d\n\n", deps.num, inst.num, wish.num);

    /* copy wish to rest */
    for(iwish = 0; iwish < wish.num; iwish++)
        str_append(&rest, word_get(&wish, iwish, 0, tmp, sizeof(tmp)));

    /* go through rest: put package to done, to impossible, or leave there */
    /* we are finished when rest does not shrink any more */
    numrest = rest.num + 1; // just to make them unequal
    while(numrest > rest.num)
    {
        numrest = rest.num;
        irest = 0;
        while(irest < rest.num)
        {
            word_get(&rest, irest, 0, pkg, sizeof(pkg));
            isdone = 1;
            isimp = 0;
            ideps = head_pos(&deps, pkg);
            depcol = 1;
            while(depword = word_get(&deps, ideps, depcol, tmp, sizeof(tmp)))
            {
                if(head_pos(&deps, depword) < 0)
                {
                    isimp = 1;
                    isdone = 0;
                    printf("unresolved: %s (misses %s)\n", pkg, depword);
                }

                if(!isimp)
                {
                    if(str_pos(&done, depword) < 0)
                    {
                        isdone = 0;
                        if(str_pos(&imp, depword) < 0)
                        {
                            pos = str_pos(&rest, depword);
                            if(pos < 0)
                            {
                                str_append(&rest, depword);
                                numrest = rest.num + 1;
                            }
                        }
                    }
                }

                depcol++;
            }
            
            if(isdone)
            {
                str_append(&done, pkg);
                str_delete(&rest, irest);
                irest--;
            }
            
            if(isimp)
            {
                str_append(&imp, pkg);
                str_delete(&rest, irest);
                irest--;
            }
            
            irest++;
        }
    }
    printf("\n");

    /* write output files */
    
    for(idone = 0; idone < done.num; idone++)
    {
        strcpy(pkg, str_get(&done, idone));
        if(head_pos(&inst, pkg) < 0)
            str_append(&result, pkg);
    }
    list_write(&result, "result.txt");

    if(rest.num)
    {
        printf("depending on unresolved packages:\n\n");
        for(irest = 0; irest < rest.num; irest++)
            printf("%s\n", str_get(&rest, irest));
        printf("\n");
    }

    printf("resolved packages:\n\n");
    for(iresult = 0; iresult < result.num; iresult++)
        printf("%s\n", str_get(&result, iresult));
    printf("\n");
    printf("%d packages resolved\n", result.num);

    /* free the list */
    list_free(&deps);
    list_free(&inst);
    list_free(&wish);
    list_free(&rest);
    list_free(&done);
    list_free(&imp);
    list_free(&result);
        
    return 0;
}
