#define _GNU_SOURCE
#include <dlfcn.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>

#define MKDIR		0
#define OPEN		1
#define SYM		2
#define MOVE		3
#define CHNG_TIME	4
#define TRUNC		5
#define MKNOD		6

#if (defined (_LARGEFILE64_SOURCE) || defined (__USE_FILE_OFFSET64))
#define LARGEFILE_SUPPORT
#endif

/* Original Glibc functions */
static int 	(*real_open) (const char *, int, ...);
static FILE* 	(*real_fopen) (const char *, const char *);
static int 	(*real_creat) (const char *, mode_t);
static int 	(*real_mkdir) (const char *, mode_t);
static int 	(*real_mknod) (const char *, mode_t, dev_t);
static int 	(*real_link) (const char *, const char *);
static int 	(*real_symlink) (const char *, const char *);
static int 	(*real_rename) (const char *, const char *);
static int 	(*real_utime) (const char *, const struct utimbuf *);
static int 	(*real_utimes) (const char *, const struct timeval *);
static int 	(*real_truncate) (const char *, off_t);

#ifdef LARGEFILE_SUPPORT
static int 	(*real_open64) (const char *, int, ...);
static FILE*    (*real_fopen64) (const char *, const char *);
static int      (*real_creat64) (const char *, mode_t);
static int      (*real_truncate64) (const char *, off_t);
#endif

char *sanity_file;

void _init ()
{
	void *lib = NULL;

	lib = RTLD_NEXT;

	/* Original functions */
	real_open = dlsym (lib, "open");
	real_fopen = dlsym (lib, "fopen");
	real_creat = dlsym (lib, "creat");
	real_mkdir = dlsym (lib, "mkdir");
	real_mknod = dlsym (lib, "mknod");
	real_link = dlsym (lib, "link");
	real_symlink = dlsym (lib, "symlink");
	real_rename = dlsym (lib, "rename");
	real_utime = dlsym (lib, "utime");
	real_utimes = dlsym (lib, "utimes");
	real_truncate = dlsym (lib, "truncate");

#ifdef LARGEFILE_SUPPORT
	real_open64 = dlsym (lib, "open64");
	real_fopen64 = dlsym (lib, "fopen64");
	real_creat64 = dlsym (lib, "creat64");
	real_truncate64 = dlsym (lib, "truncate64");
#endif

	sanity_file = (char *) malloc (255);
	sanity_file = getenv ("SANITY_FILE");
}

int str2file (char *fname, char *str)
{
	FILE *file;

	file = real_fopen (fname, "a");
	fprintf (file, "%s\n", str);
	return fclose (file);
}

void file_exists (const char *pathname)
{	
	if (real_open (pathname, O_RDONLY) != -1)
	{
		fprintf (stdout, "File already exists !\n");
		exit (1);
	}
}	

void loggomania (const char *file, const char* file2, int type)
{
	char *stuff;

	stuff = (char *) malloc (255);
		
	switch (type)
	{
		case MKDIR:
			snprintf (stuff, 255, "%s Directory", file);
			break;
		case OPEN:
			snprintf (stuff, 255, "%s File", file);
			break;
		case SYM:
			snprintf (stuff, 255, "%s Symlink to %s", file,
					file2);
			break;
		case MOVE:
			snprintf (stuff, 255, "%s Move to %s", file, 
					file2);
			break;
		case CHNG_TIME:
			snprintf (stuff, 255, "%s Touched", file);
			break;
		case TRUNC:
			snprintf (stuff, 255, "%s Truncated to %s", file,
					file2);
			break;
		case MKNOD:
			snprintf (stuff, 255, "%s Device %s", file, file2);
			break;
		default:
			stuff = "ERROR ! Unknown type.";
			break;
	}

	str2file (sanity_file, stuff);
	free (stuff);
}

/* Wrapper functions */

int mkdir (const char *pathname, mode_t mode)
{
	int ret;

	ret = real_mkdir(pathname, mode);
	if (ret) return ret;
	
	loggomania (pathname, NULL, MKDIR);
	
	return ret;
}

int open (const char *pathname, int flags, ...)
{
	va_list ap;
	mode_t mode;
	int ret;

	va_start(ap, flags);
	mode = va_arg(ap, mode_t);
	va_end(ap);
	
	ret = real_open(pathname, flags, mode);
	if (ret == -1) return -1;

	if (((flags & O_WRONLY) || (flags & O_RDWR)) && (strncmp ((char *) 
					pathname, "/dev/tty", 8)))
		loggomania (pathname, NULL, OPEN);

	return ret;
}

FILE *fopen (const char *path, const char *mode)
{
	FILE *ret;

	ret = real_fopen (path, mode);
	if (ret == NULL) return NULL;

	if (strcmp (mode, "r"))
		loggomania (path, NULL, OPEN);

	return ret;
}

int creat (const char *pathname, mode_t mode)
{
	int ret;

	ret = real_creat (pathname, mode);
	if (ret == -1) return -1;

	loggomania (pathname, NULL, OPEN);

	return ret;
}

int symlink (const char *oldpath, const char *newpath)
{
	int ret;

	ret = real_symlink (oldpath, newpath);
	if (ret) return ret;

	loggomania (oldpath, newpath, SYM);

	return ret;
}

int link (const char *oldpath, const char *newpath)
{
	return symlink (oldpath, newpath);
}

int rename (const char *oldpath, const char *newpath)
{
	int ret;

	ret = real_rename (oldpath, newpath);
	if (ret) return ret;

	loggomania (oldpath, newpath, MOVE);

	return ret;
}

int utime (const char *filename, const struct utimbuf *buf)
{
	int ret;

	ret = real_utime (filename, buf);
	if (ret == -1) return -1;

	loggomania (filename, NULL, CHNG_TIME);

	return ret;
}

int utimes (const char *filename, const struct timeval *tvp)
{
	int ret;

	ret = real_utimes (filename, tvp);
	if (ret == -1) return -1;

	loggomania (filename, NULL, CHNG_TIME);

	return ret;
}

int truncate (const char *path, off_t length)
{
	char retme[10];
	int ret;

	ret = real_truncate (path, length);
	if (ret == -1) return -1;

	snprintf (retme, 10, "%i", (int) length);
	loggomania (path, retme, TRUNC);

	return ret;
}

int mknod (const char *pathname, mode_t mode, dev_t dev)
{
	char *retme, type;
	int ret;

	ret = real_mknod (pathname, mode, dev);
	if (ret) return ret;

	if (mode & S_IFCHR)
		type = 'c';
	if (mode & S_IFBLK)
		type = 'b';
	if (mode & S_IFIFO)
		type = 'p';
	
	retme = (char *) malloc (255);
	snprintf (retme, 255, "%c %i %i", type, (int) major(dev), 
			(int) minor(dev));
	loggomania (pathname, retme, MKNOD);
	free (retme);

	return ret;
}

#ifdef LARGEFILE_SUPPORT

int open64 (const char *pathname, int flags, ...)
{
	va_list ap;
	mode_t mode;
	
	va_start(ap, flags);
	mode = va_arg(ap, mode_t);
	va_end(ap);
	
	return open (pathname, flags, mode);
}

FILE *fopen64 (const char *path, const char *mode)
{
	return fopen (path, mode);
}

int creat64 (const char *pathname, mode_t mode)
{
	return creat (pathname, mode);
}

int truncate64 (const char *path, off64_t length)
{
	return truncate (path, length);
}

#endif
